@@ -17,11 +17,11 @@ from exceptions import *
 from meta import WidgetType, WidgetsList
 
 __all__ = [
-    "Widget", 
-    "WidgetsList", 
-    "WidgetRepeater", 
+    "Widget",
+    "WidgetsList",
+    "WidgetRepeater",
     "RepeatedWidget",
-    "WidgetBunch", 
+    "WidgetBunch",
     "RepeatingWidgetBunch",
     "Child",
     "valid_id",
@@ -72,7 +72,7 @@ serial_generator = count()
 
 class Widget(view.Renderable):
     """
-    Base class for all widgets. 
+    Base class for all widgets.
 
     Example:
 
@@ -88,16 +88,16 @@ class Widget(view.Renderable):
         >>> [c.parent.id for c in w.children]
         ['foo', 'foo']
 
-    It is a **must** that all initial state is entirely determined by the 
-    arguments to this function. This means that two widgets 
+    It is a **must** that all initial state is entirely determined by the
+    arguments to this function. This means that two widgets
     (of the same class) that receive the same parameters
     must behave in exactly the same way. You should not rely on external
-    sources inside __init__ to set initial state. 
-    
-    If you need to fetch data from external sources, do it at 
+    sources inside __init__ to set initial state.
+
+    If you need to fetch data from external sources, do it at
     :meth:`update_params` instead.
 
-    Essential pre, and post initialization is done in :meth:`__new__` and 
+    Essential pre, and post initialization is done in :meth:`__new__` and
     :meth:`post_init` respectively. :meth:`post_init` is guaranteed to run after
     the instance finishes initialization and it's behavior is rather special
     as all post_init's in mro will be called to have a chance to set final
@@ -114,15 +114,15 @@ class Widget(view.Renderable):
         >>> w = Widget('foo', a=1, b=2)
         >>> w.id
         'foo'
-        >>> w.a 
+        >>> w.a
         1
         >>> w.b
         2
 
-    Basic post-initialization consists of caching required CSS and JS 
+    Basic post-initialization consists of caching required CSS and JS
     resources and setting the widget as initialized preventing further
     modification of it's attributes.
-        
+
     .. sourcecode:: python
 
         >>> w = Widget('foo', a='1', b='2')
@@ -135,7 +135,7 @@ class Widget(view.Renderable):
     should behave in a state-less way as they are shared among threads for
     multiple requests.
 
-    Per request modification of variables sent to the template should be 
+    Per request modification of variables sent to the template should be
     done inside :meth:`update_params` and all state flowing from parent to
     children should occur inside that dict.
 
@@ -144,11 +144,11 @@ class Widget(view.Renderable):
     of them) at display time to change behavior. You should try avoiding
     instantiating widgets for every request as their initialization could be
     quite expensive for heavily nested widgets.
-    
-    Request-local storage provided by the hosting framework in 
-    ``tw.framework.request`` can be used to pass state among widgets 
+
+    Request-local storage provided by the hosting framework in
+    ``tw.framework.request`` can be used to pass state among widgets
     which don't share the same root.
-    
+
     """
     __metaclass__ = WidgetType
 
@@ -161,16 +161,22 @@ class Widget(view.Renderable):
         'css_class': 'Main CSS class for this widget',
         'css_classes': 'A list with extra css classes for the widget.'
         }
-    template = None
-    engine_name = 'toscawidgets'
-    available_engines = ['toscawidgets']
     css = []
     javascript = []
     css_classes = []
 
     _is_initialized = False
     _is_locked = False
-    
+
+    def displays_on(self):
+        if self.is_root:
+            return tw.framework.default_view
+        else:
+            return self.parent.engine_name
+    displays_on = property(displays_on, doc="""\
+        Where the widget is being displayed on
+        """)
+
     #XXX: Some of these properties could be implemented as static attributes
     def id(self):
         return '_'.join(reversed(
@@ -226,14 +232,6 @@ class Widget(view.Renderable):
         Iterates a walk from this widget to the root of the tree
         """)
 
-    def displays_on(self):
-        if self.is_root:
-            return tw.framework.default_view
-        else:
-            return self.parent.engine_name
-    displays_on = property(displays_on, doc="""\
-        Where the widget is being displayed on
-        """)
 
     @property
     def id_path_elem(self):
@@ -251,7 +249,7 @@ class Widget(view.Renderable):
     def __new__(cls, id=None, parent=None, children=[], **kw):
         """
         Takes care of Widget instances creation. Should not need to be overrien
-        likely. 
+        likely.
         """
         obj = view.Renderable.__new__(cls,id,parent,children,**kw)
         # The previous version of this used the __dict__ attribute to update
@@ -266,7 +264,7 @@ class Widget(view.Renderable):
                     #skip setting the value of a read only property
                     pass
         obj.orig_kw = kw.copy()
-        
+
         if id is not None and not valid_id(id):
             raise ValueError("%r is not a valid id for a Widget"%id)
         obj._id = id
@@ -279,7 +277,7 @@ class Widget(view.Renderable):
             obj.parent = weakref.proxy(parent)
             obj.parent.children.append(obj)
 
-        # Append children passed as args or defined in the class, former 
+        # Append children passed as args or defined in the class, former
         # override later
         obj.c = obj.children = WidgetBunch()
         if isclass(children) and issubclass(children, WidgetsList):
@@ -292,7 +290,7 @@ class Widget(view.Renderable):
 
 
 
-        # Copy mutable attrs from __class__ into self, if not found in self 
+        # Copy mutable attrs from __class__ into self, if not found in self
         # set to None
         for name in chain(cls.params, ['css', 'javascript']):
             try:
@@ -300,10 +298,10 @@ class Widget(view.Renderable):
                 if isinstance(attr, (list,dict)):
                     attr = copy(attr)
                 setattr(obj, name, attr)
-            except AttributeError: 
+            except AttributeError:
                 # In case we try to set a read-only property
                 pass
-                
+
         # Initialize the static js calls list
         obj._js_calls = []
 
@@ -356,7 +354,7 @@ class Widget(view.Renderable):
     def __init__(self, id=None, parent=None, children=[], **kw):
         """Initializes a Widget instance.
 
-        `id` 
+        `id`
             The widget's id. All widgets in the same level of nested
             widgets trees should have distinct ids.
 
@@ -366,11 +364,11 @@ class Widget(view.Renderable):
             inside the parent's __init__ method.
 
         `children`
-            A list, or WidgetsList (instance or class) or any other iterable 
+            A list, or WidgetsList (instance or class) or any other iterable
             with a reference to the children this widget should have.
 
         `\*\*kw`
-            Any other extra keyword arguments for the widget. All of these will 
+            Any other extra keyword arguments for the widget. All of these will
             get bound to the Widget instance.
 
         """
@@ -385,7 +383,7 @@ class Widget(view.Renderable):
         oset.add_all(chain(*[c._resources for c in self.css]))
         oset.add_all(chain(*[c._resources for c in self.javascript]))
         oset.add_all(chain(*[c._resources for c in self.children]))
-        
+
 
 
     def post_init(self, *args, **kw):
@@ -402,7 +400,7 @@ class Widget(view.Renderable):
             self.javascript.append(
                 JSFunctionCalls(function_calls=self._js_calls)
                 )
-        self._collect_resources() 
+        self._collect_resources()
         #log.debug("Finished initializing %r", self)
         assert not self._is_initialized
         self._is_initialized = True
@@ -412,11 +410,11 @@ class Widget(view.Renderable):
 
     def walk(self, filter=None, recur_if_filtered=True):
         """
-        Does a pre-order walk on widget tree rooted at self optionally 
+        Does a pre-order walk on widget tree rooted at self optionally
         applying a filter on them.
-        
+
         Example::
-    
+
             >>> W = Widget
             >>> w = W('a', children=[W('b', children=[W('c')]), W('d')])
             >>> ''.join(i._id for i in w.walk())
@@ -430,7 +428,7 @@ class Widget(view.Renderable):
 
             >>> ''.join(i._id for i in w.walk(lambda x: x._id == 'c', False))
             ''
-            
+
         """
         def _walk():
             yield self
@@ -446,7 +444,7 @@ class Widget(view.Renderable):
 
     def add_call(self, call, location="bodybottom"):
         """
-        Adds a :func:`tw.api.js_function` call that will be made when the 
+        Adds a :func:`tw.api.js_function` call that will be made when the
         widget is rendered.
         """
         if self._is_initialized:
@@ -456,7 +454,7 @@ class Widget(view.Renderable):
         else:
             #log.debug("Adding call <%s> for %r statically.", call, self)
             self._js_calls.append(str(call))
-            
+
 
 
 
@@ -501,9 +499,9 @@ class Widget(view.Renderable):
                 else:
                     value[k] = v
         # If we have children, create a dict from the attributes of value
-        elif len(self.children_deep) > 0 and not isinstance(value, (dict,list)):
+        elif len(self.children_deep) > 0 and not isinstance(value, (dict,list, tuple)):
             value = dict([
-                (w._id, getattr(value, w._id)) 
+                (w._id, getattr(value, w._id))
                     for w in self.children_deep if w._id and hasattr(value, w._id)
                 ])
         return value
@@ -531,7 +529,7 @@ class Widget(view.Renderable):
         Renders a widget and adapts the output. This method **must** be used
         to display child widgets inside their parent's template so output is
         adapted.
-        
+
         Unlike :meth:`tw.api.Widget.render`, :meth:`tw.api.Widget.display`
         returns adapted output compatible with the template the widget is being
         rendered on. For example, this is needed so Genshi doesn't autoescape
@@ -608,17 +606,17 @@ class Widget(view.Renderable):
 
         Widget subclasses can call super cooperatively to avoid
         boiler-plate code as `Widget.update_params` takes care of pre-populating
-        this dict with all attributes from self listed at `params` 
+        this dict with all attributes from self listed at `params`
         (copying them if mutable) and preparing arguments for child widgets.
 
         Any parameter sent to `display` or `render` will override those fetched
         from the instance or the class.
 
         Any function listed at `params` which can be called without arguments
-        will be automatically called to fetch fresh results on every request. 
-        Parameters not found either on the class, the instance or the keyword 
+        will be automatically called to fetch fresh results on every request.
+        Parameters not found either on the class, the instance or the keyword
         args to `display` or `render` will be set to None.
-        
+
         .. sourcecode:: python
 
             >>> class MyWidget(Widget):
@@ -654,10 +652,6 @@ class Widget(view.Renderable):
             d[k] = attr
 
 
-
-
-
-
     def ifilter_children(self, filter):
         """
         Returns an iterator for all children applying a filter to them.
@@ -678,14 +672,10 @@ class Widget(view.Renderable):
         return ifilter(filter, self.children)
 
 
-
-
-
-
     def _get_child_value_getter(self, value):
         def value_getter(child_id):
             if value:
-                if (hasattr(child_id, 'repetition') and 
+                if (hasattr(child_id, 'repetition') and
                     isinstance(value,list)
                 ):
                     child_id = child_id.repetition
@@ -702,11 +692,11 @@ class Widget(view.Renderable):
         if isinstance(child_args, dict):
             child_args = unflatten_args(child_args)
         def args_getter(child_id):
-            if (hasattr(child_id, 'repetition') and 
+            if (hasattr(child_id, 'repetition') and
                 isinstance(child_args, list)
             ):
                 child_id = child_id.repetition
-            elif (isinstance(child_id, Widget) and 
+            elif (isinstance(child_id, Widget) and
                 isinstance(child_args, dict)
             ):
                 child_id = child_id._id
@@ -759,9 +749,9 @@ class Widget(view.Renderable):
 
     def __eq__(self, other):
         return (
-          (getattr(other, '__class__', None) is self.__class__) and 
+          (getattr(other, '__class__', None) is self.__class__) and
           # Check _id so ancestors are not taken into account
-          (other._id == self._id) and 
+          (other._id == self._id) and
           (other.children == self.children) and (other.orig_kw == self.orig_kw)
           )
 
@@ -788,7 +778,7 @@ class Child(object):
 
     Creates a Widget instance with supplied arguments to the constructor when
     called (optionally overriding default arguments).
-    
+
         >>> c = Child(Widget, 'foo')
         >>> w = c()
         >>> w.id
@@ -804,7 +794,7 @@ class Child(object):
     __slots__ = ("widget_class", "id", "children", "kw")
 
     def __init__(self, widget_class, id=None, children=[], **kw):
-        self.widget_class, self.id  = widget_class, id, 
+        self.widget_class, self.id  = widget_class, id,
         self.children, self.kw = children, kw
 
     def __call__(self, parent=None, **kw):
@@ -825,16 +815,16 @@ _WidgetBunch_attrs = frozenset(['_widget_lst', '_widget_dct', '_widget',
                                 '_repetitions', '_parent', '_repetition_cache'])
 class WidgetBunch(object):
     """
-    An ordered collection of widgets. 
+    An ordered collection of widgets.
 
-        >>> from tw.core import Widget 
+        >>> from tw.core import Widget
         >>> wb = WidgetBunch(Widget('foo'), Widget('bar'))
         >>> wb[0].id == 'foo'
         True
         >>> wb[1].id == 'bar'
         True
 
-    Exposes a mixed dict/list interface which permits indexing both by widget 
+    Exposes a mixed dict/list interface which permits indexing both by widget
     id and position.
 
         >>> wb['foo'] == wb[0]
@@ -849,7 +839,7 @@ class WidgetBunch(object):
         >>> wb.foo.id == 'foo'
         True
         >>> wb.append(Widget('append'))
-        >>> wb.append.id 
+        >>> wb.append.id
         Traceback (most recent call last):
             ...
         AttributeError: 'function' object has no attribute 'id'
@@ -986,7 +976,7 @@ class RepeatingWidgetBunch(WidgetBunch):
             reps = self._repetitions
         for i in xrange(reps):
             yield self[i]
-    
+
     def __getitem__(self, item):
         if not isinstance(item, int):
             raise KeyError("Must specify an integer")
@@ -1003,14 +993,6 @@ class RepeatingWidgetBunch(WidgetBunch):
         return rep
 
 
-
-
-
-
-
-
-
-
 class WidgetRepeater(Widget):
     params = ["repetitions", "extra"]
     widget = None
@@ -1052,13 +1034,7 @@ class WidgetRepeater(Widget):
             w.render(v_f(w), isextra=(w.repetition >= len(d['value'])), **a_f(w)) for w in d['children'].__iter__(
                 max(d['repetitions'], len(d['value']) + d.extra))
             ]
-        d["output"] = '\n'.join(o for o in outputs if o)    
-
-
-
-
-
-
+        d["output"] = '\n'.join(o for o in outputs if o)
 
 
 class RepeatedWidget(Widget):
@@ -1074,19 +1050,6 @@ class RepeatedWidget(Widget):
         return self.clone(*args, **kw)
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
 if __name__ == "__main__":
     import doctest
     doctest.testmod(optionflags = doctest.ELLIPSIS|doctest.NORMALIZE_WHITESPACE)
