@@ -56,13 +56,13 @@ class EngineManager(dict):
         self._lock = threading.Lock()
         if load_all:
             self.load_all()
-            
+
 
     def __repr__(self):
         return "< %s >" % self.__class__.__name__
 
 
-    def load_engine(self, name, options=None, extra_vars_func=None, 
+    def load_engine(self, name, options=None, extra_vars_func=None,
                     distribution=None):
         factory = None
         if distribution:
@@ -172,7 +172,7 @@ class EngineManager(dict):
             from kid import XML
             output = XML(output)
         return output
-    
+
 # Available as a public symbol for easier extension with PEAK-Rules for
 # backwards compatibility
 display = EngineManager.display.im_func
@@ -192,6 +192,7 @@ def dynamic_best_engine(renderable, params):
         ideal_engine = params['displays_on']
     except KeyError:
         ideal_engine = renderable.displays_on
+
     if ideal_engine not in renderable.available_engines:
         try:
             best_engine = params['engine_name']
@@ -201,7 +202,7 @@ def dynamic_best_engine(renderable, params):
         best_engine = ideal_engine
     assert best_engine is not None
     return best_engine
-    
+
 _is_inline_template = re.compile(r'(<|\n|\$)').search
 
 class Renderable(object):
@@ -223,7 +224,12 @@ class Renderable(object):
         obj.template = kw.pop("template", obj.template)
         engine_name = kw.pop("engine_name", None)
         if obj.template is not None:
-            engine_name, obj.template = choose_engine(obj)
+            colon_based_engine_name, obj.template = choose_engine(obj)
+            # if there is a colon in the engine name, the available engines should be narrowed
+            # to that engine.
+            if colon_based_engine_name:
+                obj.available_engines = [colon_based_engine_name,]
+            engine_name = colon_based_engine_name
         if engine_name:
             obj.engine_name = engine_name
         return obj
